package daruma.server;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.io.ByteArrayInputStream;
import java.io.PrintWriter;
import java.util.Map;
import java.util.HashMap;

import javax.xml.transform.TransformerException;
import javax.xml.parsers.ParserConfigurationException;

import daruma.sql.DatabaseConnectionException;
import daruma.util.LogWriter;
import daruma.util.PropertyReader;
import daruma.xml.util.XMLFormatConverter;
import daruma.wfs.SOAPFaultDocumentBuilder;


public class DarumaServlet extends HttpServlet
{
    private static final long serialVersionUID = 0L;

    private long connectionID = 0;

    @Override
    public void doGet( HttpServletRequest  request,
		       HttpServletResponse  response )
	throws IOException
    {
	response.setContentType( "text/xml; charset=UTF-8" );

	//
	// get parameters
	//
	String service = request.getParameter("SERVICE");
	String version = request.getParameter("VERSION");
	String req     = request.getParameter("REQUEST");
//	String ns      = request.getParameter("NAMESPACE");


	//
	// check each parameters
	//

	// check SERVICE
	if ( service == null )
	{
	    this.returnError( "no SERVICE parameter found."
			      + " add SERVICE parameter.\n"
			      + " format: SERVICE=WFS",
			      response );
	    return;
	}
	else if ( ! service.equals( "WFS" )
	       && ! service.equals( "MISP" ) )
	{
	    this.returnError( "SERVICE type [" + service + "] unsupported",
			      response );

	    return;
	}

	// check VERSION
	if ( version == null )
	{
	    if ( ! ( req != null && req.equals( "GetCapabilities" ) ) )
	    {
		this.returnError( "no VERSION parameter found."
				  + " add VERSION parameter.\n"
				  + " format: VERSION=1.1.0",
				  response );
		return;
	    }
	}
	else if ( ! version.equals( "1.1.0" ) )
	{
	    this.returnError( "VERSION [" + version + "] not supported,"
			      + " use VERSION=1.1.0",
			      response );

	    return;
	}

	if ( req == null )
	{
	    // XXX: return REQUEST candidates
	    this.returnError( "no REQUEST found."
			      + " add REQUEST parameter.",
			      response );

	    return;
	}


	String reqStr;
	if ( req.equals( "GetCapabilities" ) )
	{
	    reqStr = "<misp:GetCapabilities"
		     + " xmlns:misp=\"http://www.infosharp.org/misp\"/>";
	}
	else if ( req.equals( "GetCapabilities" )
		  || req.equals( "DescribeFeatureType" )
		  || req.equals( "Transaction" )
		  || req.equals( "GetFeature" )
		  || req.equals( "RegisterFeatureType" )
		  || req.equals( "RegisterCoordinateSystemTransformation" ) )
	{
	    this.returnError( "REQUEST [" + req + "] by http GET method"
			      + " not supported yet."
			      + " Use http POST instead",
			      response );

	    return;
	}
	else
	{
	    // XXX: return REQUEST candidates
	    this.returnError( "bad REQUEST [" + req + "]",
			      response );

	    return;
	}


	//
	// delegate to main handler
	//
	this.handleConnection( new ByteArrayInputStream( reqStr.getBytes() ),
			       response.getOutputStream() );
    }


    @Override
    public void doPost( HttpServletRequest  request,
			HttpServletResponse  response )
	throws IOException
    {
	response.setContentType( "text/xml; charset=UTF-8" );

	this.handleConnection( request.getInputStream(),
			       response.getOutputStream() );
    }


    private void handleConnection( InputStream in, OutputStream out )
							throws IOException
    {
	//
	// initialize global Classes
	//
	daruma.util.PropertyReader.initialize
	    ( this.getClass().getClassLoader().getResourceAsStream
	      ( daruma.util.PropertyReader.getDefaultPropertyFile() ) );
	daruma.util.LogWriter.initialize
	    ( this.getClass().getClassLoader().getResource
	      ( daruma.util.PropertyReader.getDefaultPropertyFile() ) );


	//
	// create connection
	//
	ConnectionHandler c;

	try
	{
	    c = new ConnectionHandler
		    ( in,
		      out,
		      PropertyReader.getProperty("daruma.db.name"),
		      PropertyReader.getProperty("daruma.db.uid"),
		      PropertyReader.getProperty("daruma.db.pass"),
		      "servlet",
		      true,
		      false,
		      null,
		      this.connectionID );

	    this.connectionID ++;
	}
	catch( DatabaseConnectionException  e )
	{
	    throw new IOException( e.getMessage() );
	}


	//
	// do main process
	//
	c.run();
    }


    private void returnError( String  errorMessage,
			      HttpServletResponse  response )
						throws IOException
    {
	try
	{
	    XMLFormatConverter.print( new SOAPFaultDocumentBuilder
					  ( errorMessage ).newDocument(),
				      response.getOutputStream() );
	}
	catch( ParserConfigurationException e )
	{
	    e.printStackTrace();
	}
	catch( TransformerException e )
	{
	    e.printStackTrace();
	}
    }
}
